import linearalgebra.GeneralSolution;
import linearalgebra.LinearSystem;
import linearalgebra.Util;
import linearalgebra.VectorSpace;
import linearalgebra.VectorSpaceInterface;
import linearalgebra.numeric.Matrix;
import linearalgebra.numeric.NumericColumn;
import linearalgebra.numeric.NumericVector;
import linearalgebra.numeric.NumericVectorInterface;

public class ExampleSet2 {
	public void example2_2_3() {
		double[][] va = {
			{1, 2, 0, 1},
			{0, 1, 1, 0},
			{1, 2, 0, 1},
		};
		Matrix a = new Matrix(va);
		// Compute an LU factorization
		LinearSystem s = new LinearSystem( a, false );
		s.solve();		
		System.out.println(s.toString());
		s.simplifyConstraints();
		System.out.println();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i));
		}
		// Determine a set of basic variables and a set of free variables
		GeneralSolution gs = s.getGeneralSolution();
		System.out.println(gs.toString());
	}
	
	public void example2_2_4() {
		double[][] va = {
			{0, 1, 4, 0},
			{0, 2, 8, 0},
		};
		Matrix a = new Matrix(va);
		LinearSystem s = new LinearSystem( a, false );
		s.solve();
		s.simplifyConstraints();
		System.out.println(s.toString());
		System.out.println();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i));
		}
		// use primes to figure out final combination
		System.out.println();
		s.setB( new NumericColumn( new double[] {2, 3} ) ); 
		s.solve();
		s.simplifyConstraints();
		System.out.println(s);
		System.out.println();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i));
		}
		int rank = s.getRank();
		GeneralSolution gs = s.getGeneralSolution();
		System.out.println();
		System.out.println(gs);		
		System.out.println("Rank is "+rank);
	}

	public void example2_2_5() {
		double[][] va = {
			{0, 0},
			{1, 2},
			{4, 8},
			{0, 0},
		};
		Matrix a = new Matrix(va);
		LinearSystem s = new LinearSystem( a, false );
		s.solve();
		s.simplifyConstraints();
		System.out.println(s.toString());
		System.out.println();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i));
		}
		System.out.println();
		// use primes to figure out final combination
		s.setB( new NumericColumn( new double[] {2, 3, 5, 7} ) ); 
		s.solve();
		s.simplifyConstraints();
		System.out.println(s);
		System.out.println();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i));
		}
		int rank = s.getRank();
		GeneralSolution gs = s.getGeneralSolution();
		System.out.println();
		System.out.println(gs);		
		System.out.println("Rank is "+rank);
	}

	public void example2_2_6() {
		double[][] va = {
			{1, 2, 2},
			{2, 4, 5},
		};
		LinearSystem s = new LinearSystem( new Matrix(va), new NumericColumn( new double[] {1, 4} ), false );
		s.solve();
		s.simplifyConstraints();
		GeneralSolution gs = s.getGeneralSolution();
		System.out.println(s);
		System.out.println(gs);		
	}

	public void example2_2_10() {
		double[][] va = {
			{1, 0, 0},
			{0, 1, 0},
			{0, 0, 1},
		};
		LinearSystem s = new LinearSystem( new Matrix(va), false );
		s.solve();
		s.simplifyConstraints();
		GeneralSolution gs = s.getGeneralSolution();
		System.out.println(s);
		System.out.println(gs);	
	}
	
	public void example2_3_1() {
		NumericVectorInterface[] cols = {
			new NumericVector( new double[] {1,1,0,0} ),
			new NumericVector( new double[] {1,0,1,0} ),
			new NumericVector( new double[] {0,0,1,1} ),
			new NumericVector( new double[] {0,1,0,1} ),
		};
		VectorSpace space = new VectorSpace( cols );
		System.out.println("Vectors are independent: " + space.areIndependentColumns() );
	}
	
	public void example2_3_2() {
		NumericVectorInterface[] cols = {
			new NumericVector( new double[] {1,1,2} ),
			new NumericVector( new double[] {1,2,1} ),
			new NumericVector( new double[] {3,1,1} ),
		};
		VectorSpace space = new VectorSpace( cols );
		System.out.println("Vectors are independent: " + space.areIndependentColumns() );
	}
	
	public void example2_3_7() {
		NumericVectorInterface[] ac = {
			new NumericVector( new double[] {1,1,0} ),
			new NumericVector( new double[] {2,2,1} ),
			new NumericVector( new double[] {0,0,2} ),
		};
		VectorSpace a = new VectorSpace( ac );
		System.out.println("Space a constains b: " + a.isMember(new NumericVector(new double[] { 3, 4, 5 })) );
		
		NumericVectorInterface[] bc = {
			new NumericVector( new double[] {1,2,0} ),
			new NumericVector( new double[] {2,5,0} ),
			new NumericVector( new double[] {0,0,2} ),
			new NumericVector( new double[] {0,0,0} ),
		};
		VectorSpace b = new VectorSpace( bc );
		System.out.println("Space b constains b1: " + b.isMember(new NumericVector(new double[] { 0,0,0 })) );
		System.out.println("Space b constains b2: " + b.isMember(new NumericVector(new double[] { 3,5,7 })) );
		System.out.println("Space b constains b3: " + b.isMember(new NumericVector(new double[] { 17,13,11 })) );
	}
	
	public void example2_3_8() {
		double va[][] = new double[][] {
			{1, 2},
			{3, 6},
		};
		Matrix a = new Matrix(va);
		VectorSpaceInterface space = new VectorSpace(a);
		VectorSpaceInterface colSpace = space.getColumnSpace();
		VectorSpaceInterface rowSpace = space.getRowSpace();
		System.out.println(space);
		System.out.println(colSpace);
		System.out.println(rowSpace);
	}
	
	public void example2_3_9() {
		double va[][] = new double[][] {
			{0,1,4,3},
			{0,0,2,2},
			{0,0,0,0},
			{0,0,0,0},
		};
		Matrix a = new Matrix(va);
		VectorSpaceInterface space = new VectorSpace(a);
		VectorSpaceInterface colSpace = space.getColumnSpace();
		VectorSpaceInterface rowSpace = space.getRowSpace();
		System.out.println(colSpace);
		System.out.println(rowSpace);
		int[] cols = space.getBasisColumns();
		int pivotCol = 0;
		for( int i = 0; i < a.getColumnCount(); i++ ) {
			if( pivotCol < cols.length && cols[pivotCol] == i ) {
				pivotCol++;
			} else {
				LinearSystem s = new LinearSystem( colSpace.getMatrix(), a.getNumericColumnView(i), false );
				System.out.println(s);
				s.solve();
				s.simplifyConstraints();
				System.out.println(s.getGeneralSolution());
			}
		}
	}
	
	public void example2_4_2() {
		double va[][] = new double[][] {
			{0,1,4,0},
			{0,2,8,0},
		};
		Matrix U = new Matrix(va);
		Matrix L = (Matrix)U.factorL(null, false);
		assert(L.product(U).equals(new Matrix(va)));
		
		Matrix a = new Matrix(va);
		VectorSpaceInterface space = new VectorSpace(a);
		VectorSpaceInterface rowSpace = space.getRowSpace();
		VectorSpaceInterface nullSpace = space.getNullspace();
		VectorSpaceInterface colSpace = space.getColumnSpace();
		VectorSpaceInterface leftNullSpace = space.getLeftNullspace();
		System.out.println(space);
		System.out.println(colSpace);
		System.out.println(rowSpace);
		System.out.println(leftNullSpace);
		System.out.println(nullSpace);
		assert( colSpace.toString().equals("1.0\t\n2.0\t\n") );
		assert( rowSpace.toString().equals("0.0\t1.0\t4.0\t0.0\t\n") );
		assert( leftNullSpace.toString().equals("-2.0\t\n1.0\t\n") );
		assert( nullSpace.toString().equals("1.0\t0.0\t0.0\t\n0.0\t-4.0\t0.0\t\n0.0\t1.0\t0.0\t\n0.0\t0.0\t1.0\t\n") );
	}
	
	public void example2_4_3() {
		double va[][] = new double[][] {
			{1,2,0,1},
			{0,1,1,0},
			{1,2,0,1},
		};		
		Matrix a = new Matrix(va);
		VectorSpaceInterface space = new VectorSpace(a);
		VectorSpaceInterface colSpace = space.getColumnSpace();
		VectorSpaceInterface rowSpace = space.getRowSpace();
		VectorSpaceInterface leftNullSpace = space.getLeftNullspace();
		VectorSpaceInterface nullSpace = space.getNullspace();
		System.out.println(space);
		System.out.println(colSpace);
		System.out.println(rowSpace);
		System.out.println(leftNullSpace);
		System.out.println(nullSpace);
		assert( colSpace.toString().equals("1.0\t2.0\t\n0.0\t1.0\t\n1.0\t2.0\t\n") );
		assert( rowSpace.toString().equals("1.0\t2.0\t0.0\t1.0\t\n0.0\t1.0\t1.0\t0.0\t\n") );
		assert( leftNullSpace.toString().equals("-1.0\t\n0.0\t\n1.0\t\n") );
		assert( nullSpace.toString().equals("2.0\t-1.0\t\n-1.0\t0.0\t\n1.0\t0.0\t\n0.0\t1.0\t\n") );
	}
	
	public void example2_5_1() {
		NumericVector x = new NumericVector(new double[] { 1, 4,0,2	});		
		NumericVector y = new NumericVector(new double[] { 2,-2,1,3	});
		double lx = x.getLength();
		double ly = y.getLength();
		double ip = x.getInnerProduct(y);
		System.out.println(lx+" "+ly+" "+ip);
		assert(lx==Math.sqrt(21));
		assert(ly==Math.sqrt(18));
		assert(ip==0.0);	
	}
	
	public void example2_5_5() {
		NumericVector v1 = new NumericVector(new double[] { 1, 2,-2, 1	});		
		NumericVector v2 = new NumericVector(new double[] { 4, 0, 4, 0	});
		NumericVector v3 = new NumericVector(new double[] { 1,-1,-1,-1	});
		System.out.println("v1_|_v2 = " + v1.isOrthogonal(v2) );
		System.out.println("v1_|_v3 = " + v1.isOrthogonal(v3) );
		System.out.println("v2_|_v3 = " + v2.isOrthogonal(v3) );
		assert(!v1.isOrthogonal(v2));
		assert(v1.isOrthogonal(v3));
		assert(v2.isOrthogonal(v3));
	}
		
	public void example2_5_6() {
		double va[][] = new double[][] {
			{1, 1, 1},
			{1,-1, 0},
		};
		LinearSystem l = new LinearSystem( new Matrix(va) );
		l.solve();
		System.out.println(l.getGeneralSolution());
		assert(l.getGeneralSolution().toString().equals("  -0.5\nc*-0.5\n  1.0\n"));
	}
	
	public static void main( String[] args ) {		
		ExampleSet2 ex = new ExampleSet2();		
		Util.setTolerance(0.0);
/*		
		ex.example2_2_3();
		ex.example2_2_4();
		ex.example2_2_5();
		ex.example2_2_6();
		ex.example2_2_10();
		ex.example2_3_1();
		ex.example2_3_2();
		ex.example2_3_7();
		ex.example2_3_8();
		ex.example2_3_9();
		ex.example2_4_2();
		ex.example2_4_3();
		ex.example2_4_4();
		ex.example2_5_1();
		ex.example2_5_5();
*/		
		ex.example2_5_6();
		
		System.exit(0);
	}
}