import linearalgebra.LinearSystem;
import linearalgebra.Util;
import linearalgebra.numeric.Matrix;
import linearalgebra.numeric.NumericColumn;
import linearalgebra.numeric.NumericColumnInterface;
import linearalgebra.numeric.NumericMatrixInterface;
import linearalgebra.numeric.NumericVector;
import linearalgebra.numeric.NumericVectorInterface;

public class ExampleSet3 {
	
	public void exercise3_1_3() {
		NumericVector a = new NumericVector( new double[] {1,1,1} );
		NumericVector b = new NumericVector( new double[] {2,4,4} );
		NumericVector p1 = (NumericVector)a.clone();
		NumericVector p2 = (NumericVector)b.clone();
		p1.projectionOf(b);
		p2.projectionOf(a);
		System.out.println(p1);
		System.out.println(p2);
		Util.setTolerance(0.00000000000001);
		assert(Util.isEqual(p1.getValue(0), 10.0/3.0));
		assert(Util.isEqual(p1.getValue(1), 10.0/3.0));
		assert(Util.isEqual(p1.getValue(2), 10.0/3.0));
		assert(Util.isEqual(p2.getValue(0), 20.0/36.0));
		assert(Util.isEqual(p2.getValue(1), 40.0/36.0));
		assert(Util.isEqual(p2.getValue(2), 40.0/36.0));
	}
	
	public void exercise3_2_1() {
		NumericColumn a = new NumericColumn( new double[] {1,1,1,1} );
		NumericColumn b = new NumericColumn( new double[] {150,153,150,151} );
		LinearSystem s = new LinearSystem(a,b,false);
		NumericColumnInterface x = s.getLeastSquaresSolution();
		System.out.println(x.getRowValue(0));
		assert(s.getSolution(0).equals("a = 151.0"));
	}
	
	public void exercise3_2_2() {
		NumericColumn a = new NumericColumn( new double[] {3,4} );
		NumericColumn b = new NumericColumn( new double[] {10,5} );
		LinearSystem s = new LinearSystem(a,b,false);
		NumericColumnInterface x = s.getLeastSquaresSolution();
		System.out.println(x.getRowValue(0));
		assert(s.getSolution(0).equals("a = 2.0"));
	}
	
	public void exercise3_2_3() {
		{
			Matrix a = new Matrix( new double[][] {
				{1,0},
				{0,1},
				{1,1},
			});
			NumericColumn b = new NumericColumn( new double[] {1,1,0} );
			LinearSystem s = new LinearSystem(a,b,false);
			NumericColumnInterface x = s.getLeastSquaresSolution();
			for( int i = 0; i < x.getRowCount(); i++ ) {
				System.out.println(x.getRowValue(i));
			}
			assert(Util.isEqual(x.getRowValue(0), 1.0/3.0));
			assert(Util.isEqual(x.getRowValue(1), 1.0/3.0));
		}
		{
			NumericColumn a = new NumericColumn( new double[] {1,1,1} );
			NumericColumn b = new NumericColumn( new double[] {1,3,5} );
			LinearSystem s = new LinearSystem(a,b,false);
			NumericColumnInterface x = s.getLeastSquaresSolution();
			System.out.println(s.getSolution(0));
			assert(s.getSolution(0).equals("a = 3.0"));
		}
	}
	
	public void exercise3_2_4() {
		Matrix a = new Matrix( new double[][] {
			{1,0},
			{0,1},
			{1,1},
		});
		NumericColumn b = new NumericColumn( new double[] {1,3,4} );
		LinearSystem s = new LinearSystem(a,b,false);
		NumericColumnInterface x = s.getLeastSquaresSolution();
		NumericColumnInterface p = s.getLeastSquaresProjection();
		NumericVectorInterface BMinusP = b.getNumericVectorView().minus(p.getNumericVectorView());		
		System.out.println(x);
		System.out.println(p);
		for( int i = 0; i < a.getColumnCount(); i++ ) {
			assert( a.getNumericColumnView(i).getNumericVectorView().isOrthogonal(BMinusP) );
		}
	}
		
	public void exercise3_2_5() {
		Matrix a = new Matrix( new double[][] {
			{1,0},
			{0,1},
			{1,1},
		});
		NumericColumn b = new NumericColumn( new double[] {0,2,1} );
		NumericMatrixInterface p = (NumericMatrixInterface)a.getProjectionMatrix();
		NumericMatrixInterface pT = new Matrix(p.getRowCount()).minus(p);
		NumericMatrixInterface pb = p.product(b);
		
		System.out.println(p);
		System.out.println(pb);
		
		assert( p.equals( new Matrix( new double[][] {{2,1,1},{1,2,-1},{1,-1,2}} ).multiplyEquals(1.0/3.0) ) );
		assert( pb.equals( new NumericColumn( new double[] {1,1,0} ) ) );
	}
	
	public static void main( String[] args ) {
		ExampleSet3 ex = new ExampleSet3();		
		Util.setTolerance(0.0);
/*		
		ex.exercise3_1_3();
		ex.exercise3_2_1();
		ex.exercise3_2_2();
		ex.exercise3_2_3();
		ex.exercise3_2_4();
*/		
		ex.exercise3_2_5();

		System.exit(0);
	}
}