package linearalgebra;

import linearalgebra.exception.NoPivotError;

public class LinearEquation {
	private LinearExpression m_lhs = new LinearExpression();
	private LinearExpression m_rhs = new LinearExpression();
	
	public void addLeft( double coefficient, UnknownValue variable ) {
		m_lhs.add(coefficient, variable);
	}
	
	public void addRight( double value ) {
		m_rhs.add(value);
	}
	
	public LinearExpression getRHS() {
		return m_rhs;
	}
	
	public void solveFor( UnknownValue variable ) {
		double c = m_lhs.getCoefficient( variable );
		if( c == 0.0 ) {
			throw new NoPivotError();
		}
		m_lhs.remove( variable );
		m_lhs.scaleAll(-1.0);
		m_rhs.add( m_lhs );
		m_rhs.scaleAll( 1.0 / c );
		m_lhs = new LinearExpression();
		m_lhs.add( 1.0, variable );
	}
	
	public String toString() {
		return m_lhs.toString() + " = " + m_rhs.toString();
	}
}