package linearalgebra;

public interface MatrixInterface {
	public int getColumnCount();
	public int getRowCount();
	
	// operations for eliminations
	public void swapRows( int row1, int row2 );
	public void swapRows( PermutationMatrix p );
	public boolean hasRowPivot( int row );
	public boolean hasColumnPivot( int col );
	public int getRowLeftPivotIndex( int row );
	public int getRowRightPivotIndex( int row );
	public int getColumnTopPivotIndex( int col );
	public int getColumnBottomPivotIndex( int col );
//	public int getRowPivotIndex( int row ) throws linearalgebra.exception.NoPivotError ;
	
	// deconstruction operations
//	public ColumnInterface getColumnView( int colIndex );
//	public RowInterface getRowView( int rowIndex );
	
	// factoring
	public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot);
	public MatrixInterface factorL(PermutationMatrix p);	// does partial pivot
	public MatrixInterface factorU(PermutationMatrix p);
	public MatrixInterface factorInverse() 
		throws linearalgebra.exception.UninvertibleMatrixException;
		// if factorInverse throws an exception, the original matrix
		// may be left in an undefined state
	public MatrixInterface getTranspose();
	public MatrixInterface getProjectionMatrix();
	public MatrixInterface getOrthogonalProjectionMatrix();
	public MatrixInterface getReflectionMatrix();
	
	// properties
	public boolean isIdentity();
	public boolean isUpperRightTriangular();
	public boolean isLowerLeftTriangular();
	public boolean isSymmetric();
	public boolean isPositiveDefinite();
	public boolean isIdempotent();
	public boolean isOrthogonal();

	public boolean equals( Object o );
}