package linearalgebra.graph;

import java.awt.*;
import java.awt.geom.*;

public class Viewport {
	Rectangle2D m_viewport = new Rectangle2D.Double();
	public void setViewport( double xOrigin, double yOrigin, double width, double height ) {
		m_viewport.setFrame(xOrigin, yOrigin, width, height);
	}
	
	public void setViewport( Rectangle2D view ) {
		setViewport( view.getX(), view.getY(), view.getWidth(), view.getHeight() );
	}
	
	public Rectangle2D getViewport() {
		return (Rectangle2D)m_viewport.clone();
	}
	
	public Point2D viewToWorld(Point p, Dimension viewSize) {
		try {
			return getViewportTransform(viewSize).inverseTransform( new Point2D.Double(p.x, p.y), null);
		} catch( NoninvertibleTransformException e ) {
			return new Point2D.Double(p.x, p.y);
		}
	}
	
	public Point worldToView( Point2D p, Dimension viewSize ) {
		Point2D p2 = getViewportTransform(viewSize).transform(p, null);
		return new Point((int)p2.getX(), (int)p2.getY());
	}	
	
	private AffineTransform getViewportTransform(Dimension viewSize) {
		AffineTransform at = new AffineTransform();
		at.translate( 0, viewSize.height );
		at.scale( 1, -1 );
		at.scale(viewSize.width/m_viewport.getWidth(), viewSize.height/m_viewport.getHeight());
		at.translate( -m_viewport.getX(), -m_viewport.getY() );
		return at;
	}
	
	private void setViewportTransform( AffineTransform t, Dimension viewSize ) {
		Point2D p1 = new Point2D.Double( 0, viewSize.height ); // user lower left
		Point2D p2 = new Point2D.Double( viewSize.width, 0 );	// user upper right
		try {
			t.inverseTransform( p1, p1 );
			t.inverseTransform( p2, p2 );			
		} catch( NoninvertibleTransformException e ) {
		}
//		System.out.println("New viewport: " + p1.getX() + ", " + p1.getY() + ", " + p2.getX() + ", " + p2.getY() );
		setViewport( p1.getX(), p1.getY(), p2.getX()-p1.getX(), p2.getY()-p1.getY() );
	}
}