package linearalgebra.numeric;

import linearalgebra.*;

public class NumericVector extends AbstractNumericVector {
	protected double[] m_vector = {};
	
	public NumericVector( int dim ) {
		m_vector = new double[dim];
	}
	
	public NumericVector( double[] copy ) {
		m_vector = copy.clone();
	}
	
	public NumericVector( VectorInterface copy ) {
		m_vector = new double[copy.getDimension()];
		for( int i=0; i < m_vector.length; i++ ) {
			m_vector[i] = copy.getValue(i);
		}
	}
	
	public final Object clone() {
		NumericVector v = (NumericVector)super.clone();
		v.m_vector = m_vector.clone();
		return v;
	}
	
	public final double[] toArray() {
		return m_vector.clone();
	}
	
    public final int getDimension() {
        return m_vector.length;
    }
    
    public final void setValue( int index, double value ) {
    	m_vector[index] = value;
    }
    
	public void setValue(NumericVectorInterface v) {
		if( v.getDimension() != getDimension() ) {
			throw new linearalgebra.exception.SizeMismatchError("Vector sizes do not match. Vector cannot copy values.");
		}
		if( v instanceof NumericVector ) {
			NumericVector nv = (NumericVector)v;
			System.arraycopy( nv.m_vector, 0, m_vector, 0, getDimension() );
		} else {
			for( int i = 0; i < getDimension(); i++ ) {
				setValue(i, v.getValue(i));
			}
		}
	}
	
    public final double getValue( int index ) {
    	return m_vector[index];
    }
    
    public String toString() {
    	StringBuffer rc = new StringBuffer();
    	rc.append("{ ");
    	for( int i = 0; i < getDimension(); i++ ) {
    		rc.append(getValue(i));
    		if( i < getDimension() ) {
    			rc.append(",\t");
    		}
    	}
    	rc.append(" }");
    	return rc.toString();
    }
}