package linearalgebra.numeric;

import linearalgebra.numeric.*;
import linearalgebra.exception.UnsupportedOperationError;

import linearalgebra.*;
public class TransposeWrapper extends linearalgebra.numeric.AbstractMatrix {
	private NumericMatrixInterface m_matrix;
	private PermutationMatrix m_rowMap;
	
	public TransposeWrapper(NumericMatrixInterface wrapTarget) {
		super(wrapTarget.getColumnCount(), wrapTarget.getRowCount());
		m_matrix = wrapTarget;
		m_rowMap = new PermutationMatrix(getRowCount());
	}
	
	public NumericMatrixInterface getTarget() {
		return m_matrix;
	}
	
	public Object clone() {
		MatrixInterface rc = m_matrix.getTranspose();
		rc.swapRows(m_rowMap);
		return rc;
    }
	
    public double getElement(int row, int col) {
        return m_matrix.getElement(col, getRowMapping( row ));
    }

    public MatrixInterface getTranspose() {
    	return (MatrixInterface)m_matrix.clone();
    }

    public void incrementElement(int row, int col, double value) {
    	m_matrix.incrementElement( col, getRowMapping( row ), value );
    }

    public void scaleElement(int row, int col, double scalar){
    	m_matrix.scaleElement( col, getRowMapping( row ), scalar );
    }

    public void setElement(int row, int col, double element) {
    	m_matrix.setElement( col, getRowMapping( row ), element );
    }

    public void swapRows(PermutationMatrix p) {
    	m_rowMap.swapRows(p);
    }

	private int getRowMapping( int row ) {
		return m_rowMap.getRowTranspose(row);
	}
	
    public boolean isIdempotent() {
    	throw new UnsupportedOperationError();
    }
}